import click
from src.utils.utils import send_get, send_put
import logging
from os import sys

log = logging.getLogger("immich-tools")

@click.command()
@click.argument("album_id")
@click.option('-k', '--api-key', required=True, envvar="IMMICH_API_KEY")
@click.option('-u', '--url', required=True, envvar="IMMICH_URL")
@click.option('-t', '--tag', "expected_tags", required=True, multiple=True, help="expected tags (use multiple -t/--tag)")
@click.option('-a', '--add-missing', is_flag=True, help="will add missing tags")
def check_album_tags(album_id, api_key, url, expected_tags, add_missing):
    """looking in album for specified tags and return assets without expected tags"""
    log.debug(f"immich url: {url}")
    album_response = send_get(path=f"/api/albums/{album_id}", url=url, api_key=api_key).json()
    album_name = album_response["albumName"]
    log.debug(f'found album: {album_name}')
    assets = album_response["assets"]
    for asset in assets:
        asset_id = asset["id"]
        asset_info = send_get(path=f"/api/assets/{asset_id}", url=url, api_key=api_key).json()
        tags = [tag["value"] for tag in asset_info["tags"]]
        if not set(expected_tags).issubset(tags):
            missing_tags = list(set(expected_tags) - set(tags))
            print(f"asset: {asset_id}, actual tags: {tags}, missing tags: {missing_tags}")
            if add_missing:
                all_tags = __get_all_tags_value_id(url, api_key)
                __add_missing_tags(asset_id, missing_tags, url, api_key, all_tags)
    log.info("success")

def __get_all_tags_value_id(url:str, api_key: str) -> dict:
    all_tags = send_get(path="/api/tags", url=url, api_key=api_key).json()
    result = {}
    for tag in all_tags:
        result[tag["value"]] = tag["id"]
    return result

def __add_missing_tags(asset_id: str, missing_tags: list[str], url: str, api_key:str, all_tags: dict):
    for missing_tag in missing_tags:
        log.debug(f"adding tag: {missing_tag} to asset: {asset_id}")
        if missing_tag not in all_tags:
            log.error(f"there is not added tag \"{missing_tag}\" in immich. Add tag then retry.")
            sys.exit(-1)
        id = all_tags[missing_tag]
        send_put(path=f"/api/tags/{id}/assets", url=url, api_key=api_key, data={"ids": [asset_id]})


import click
import os
from os import sys
import logging
import exiftool
from exiftool.exceptions import ExifToolExecuteError

log = logging.getLogger("immich-tools")

IMAGE_EXTENSIONS = {".mp4",".jpg", ".jpeg", ".png", ".tiff", ".bmp", ".gif", ".heic", ".webp", ".mov", ".wmv"}

def find_matches(path: str) -> list[tuple[str, str]]:
    matches = []
    log.info(f"Looking for XMP files in: {path}")
    
    # Traverse the directory tree
    for dirpath, _, filenames in os.walk(path):
        for filename in filenames:
            filepath = os.path.join(dirpath, filename)
            _, ext = os.path.splitext(filepath)
            if ext.lower() in IMAGE_EXTENSIONS:
                xmp_counterpart = f"{filepath}.xmp"
                if os.path.exists(xmp_counterpart):
                    log.debug(f"file {xmp_counterpart} exists")
                    matches.append((filepath, xmp_counterpart))
    return matches


@click.command()
@click.argument("path")
@click.option("-x", "--leave-xmp", is_flag=True, help="use to leave xmp files and do not remove them")
@click.option("-o", "--leave-original", is_flag=True, help="will create copy of original file with '_original' suffix")
@click.option("--dry-run", is_flag=True, help="will run without modifying or removing any file")
def merge_xmp(path: str, leave_xmp: bool, leave_original: bool, dry_run: bool):
    """Merge xmp sidecards to original image file from PATH recursively"""
    overwrite_original = "-overwrite_original"
    if leave_original:
        overwrite_original = ""
        log.debug("option -o/--leave-original is turned on - will create copy of original file with '_original' suffix")
    if leave_xmp:
        log.debug("option -x/--leave-xmp is turned on - will not remove xmp sidecards")
    matches = find_matches(path)
    for image, xmp in matches:
        with exiftool.ExifToolHelper() as ex:
            log.debug(f"merging {xmp} with {image}")
            try:
                if dry_run:
                    log.info(f"[DRY-RUN] writing tags from {xmp} to {image}")
                else:
                    ex.execute("-tagsfromfile", f"{xmp}", "-all:all", f"{overwrite_original}", f"{image}")
                if not leave_xmp:
                    if dry_run:
                        log.info(f"[DRY-RUN] removing {xmp}")
                    else:
                        log.debug(f"removing: {xmp}")
                        os.remove(xmp)
            except ExifToolExecuteError as er:
                log.error(ex.last_stderr.strip())
                log.error(f"error while executing exiftool command: {er}")
                sys.exit(-1)
    log.info(f"merged {len(matches)} files")



import click
from src.commands import merge_xmp, refresh_album_metadata, run_job, version, check_album_tags
import logging
from datetime import datetime
import src


log = logging.getLogger("immich-tools")
log.setLevel(logging.INFO)
formatter = logging.Formatter("%(asctime)s - %(levelname)-5s - %(message)s")


console_handler = logging.StreamHandler()
console_handler.setFormatter(formatter)

log.addHandler(console_handler)


@click.group(help=f"""This is immich-tools version {src.__version__}""")
@click.option("-d", "--debug", is_flag=True, help="Enable debug mode")
@click.option("-l", "--log-file", is_flag=True, help="Saves logs to file")
@click.option("--log-path", help="path to directory where logs will be stored, works only if --log-file flag is set to True")
@click.pass_context
def main(ctx, debug:bool, log_file: bool, log_path: str):
    f"""Tools for immich, version """ 
    ctx.ensure_object(dict)
    log.info("Running immich-tools")
    if log_file:
        log_directory = log_path if log_path else "." 
        file_handler = logging.FileHandler(datetime.now().strftime(f'{log_directory}/log_%d_%m_%Y.log'))
        file_handler.setLevel(logging.DEBUG if debug else logging.INFO)
        file_handler.setFormatter(formatter)
        log.addHandler(file_handler)
    if debug:
        log.setLevel(logging.DEBUG)
        log.debug("Debug mode is ON")
    
        
main.add_command(refresh_album_metadata.refresh_album_metadata)
main.add_command(merge_xmp.merge_xmp)
main.add_command(run_job.run_job)
main.add_command(version.version)
main.add_command(check_album_tags.check_album_tags)


import click
from src.utils.utils import send_get, send_post
import logging

log = logging.getLogger("immich-tools")

@click.command()
@click.argument("album_id")
@click.option('-k', '--api-key', required=True, envvar="IMMICH_API_KEY")
@click.option('-u', '--url', required=True, envvar="IMMICH_URL")
def refresh_album_metadata(album_id, api_key, url):
    """Refresh metada in all assets in album"""
    log.debug(f"immich url: {url}")
    album_response = send_get(path=f"/api/albums/{album_id}", url=url, api_key=api_key).json()
    album_name = album_response["albumName"]
    log.debug(f'found album: {album_name}')
    assets = album_response["assets"]
    asset_ids = list()
    for asset in assets:
        asset_ids.append(asset["id"])
        data = {
            "assetIds": asset_ids,
            "name": "refresh-metadata"
        }
    log.debug(f"refreshing metadata for {len(assets)} assets")
    send_post("/api/assets/jobs", url, api_key, data)
    log.info("success")

